package xyz.twovb.sgm.games.impl.capturethebrick;

/*
 * Created by 2vb - 5/7/2024
 */

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.Style;
import net.kyori.adventure.text.format.TextDecoration;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.kyori.adventure.text.minimessage.tag.resolver.TagResolver;
import net.megavex.scoreboardlibrary.api.sidebar.Sidebar;
import net.megavex.scoreboardlibrary.api.sidebar.component.ComponentSidebarLayout;
import net.megavex.scoreboardlibrary.api.sidebar.component.SidebarComponent;
import net.megavex.scoreboardlibrary.api.sidebar.component.animation.CollectionSidebarAnimation;
import net.megavex.scoreboardlibrary.api.sidebar.component.animation.SidebarAnimation;
import net.megavex.scoreboardlibrary.api.team.ScoreboardTeam;
import net.megavex.scoreboardlibrary.api.team.TeamDisplay;
import net.megavex.scoreboardlibrary.api.team.TeamManager;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.scheduler.BukkitTask;
import org.jetbrains.annotations.NotNull;
import xyz.twovb.sgm.SGM;
import xyz.twovb.sgm.games.GameManager;
import xyz.twovb.sgm.games.Minigame;
import xyz.twovb.sgm.levels.LevelManager;
import xyz.twovb.toolbox.utils.ChatUtils;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class CTB implements Minigame {

    final static String name = "CaptureTheBrick";
    private final UUID uuid;
    private final List<Player> players;
    private final ArrayList<Location> redBrickSpawns = new ArrayList<Location>();
    private final ArrayList<Location> blueBrickSpawns = new ArrayList<Location>();
    private final ArrayList<Location> redSpawnArea = new ArrayList<Location>();
    private final ArrayList<Location> blueSpawnArea = new ArrayList<Location>();
    private final FileConfiguration messages;
    private World gameWorld;
    private GameState state;
    private int bricks;
    private int territoryLevel;
    private int boundsLevel;
    private Location spawnLoc;
    private TeamManager teamManager;
    private ScoreboardTeam redTeam;
    private ScoreboardTeam blueTeam;
    private Sidebar sb;
    private SidebarAnimation<Component> titleAnimation;
    //    private @NotNull SidebarComponent title;
    private ComponentSidebarLayout layout;

    private int taskId;

//    private List<Player> redTeam = new ArrayList<Player>();
//    private List<Player> blueTeam = new ArrayList<Player>();
//    private Sidebar sb;


    public CTB() {
        this.uuid = UUID.randomUUID();
        this.state = GameState.PRESTART;
        this.players = new ArrayList<>();
        this.messages = SGM.getInstance().getConfigManager().loadConfig("games/" + name.toLowerCase() + "/messages.yml");
    }

    public void init(CommandSender owner, String world) throws IOException {
        // Perform initialization logic here
        gameWorld = createGameWorld(world, uuid);
        gameWorld.setAutoSave(false);
        if (applyOptions(world)) {
            teamManager = SGM.getInstance().getScoreboardLibrary().createTeamManager();
            this.redTeam = teamManager.createIfAbsent("red_team");
            this.blueTeam = teamManager.createIfAbsent("blue_team");
            Bukkit.getPluginManager().registerEvents(this, SGM.getInstance());

            setupTeamDisplay(redTeam, "Red Team", NamedTextColor.RED);
            setupTeamDisplay(blueTeam, "Blue Team", NamedTextColor.BLUE);

            this.sb = SGM.getInstance().getScoreboardLibrary().createSidebar();
            this.titleAnimation = createGradientAnimation(Component.text("Capture The Brick", Style.style(TextDecoration.BOLD)));
            var title = SidebarComponent.animatedLine(titleAnimation);

            SidebarComponent gameInfo = SidebarComponent.builder().addDynamicLine(() -> Component.text("Players Alive: " + getAlivePlayersCount(), NamedTextColor.YELLOW)).addDynamicLine(() -> Component.text("Red Team: " + getTeamSize(redTeam), NamedTextColor.RED)).addDynamicLine(() -> Component.text("Blue Team: " + getTeamSize(blueTeam), NamedTextColor.BLUE)).addDynamicLine(() -> Component.text("Bricks Remaining: " + bricks, NamedTextColor.GREEN)).build();

            this.layout = new ComponentSidebarLayout(title, gameInfo);
            layout.apply(sb);

//            buildScoreboard();
            // Ready
            state = GameState.READY;
            owner.sendMessage(ChatUtils.translate(messages.getString("system.game.ready").replace("%game%", name)));
        } else {
            owner.sendMessage(ChatUtils.translate(messages.getString("system.game.failed").replace("%cause%", "to load options.")));
            this.stop();
        }
    }

    private void setupTeamDisplay(ScoreboardTeam team, String displayName, NamedTextColor color) {
        TeamDisplay display = team.defaultDisplay();
        display.displayName(Component.text(displayName));
        display.playerColor(color);
    }

//    private void buildScoreboard() {
//        titleAnimation = createGradientAnimation(Component.text("Capture The Brick", Style.style(TextDecoration.BOLD)));
//        title = SidebarComponent.animatedLine(titleAnimation);
//        SidebarComponent lines = SidebarComponent.builder().addDynamicLine(() -> Component.text("Red Team: " + redTeam.teamManager().players().size(), NamedTextColor.RED)).addDynamicLine(() -> Component.text("Blue Team: " + blueTeam.teamManager().players().size(), NamedTextColor.BLUE)).addDynamicLine(() -> Component.text("Bricks Remaining: " + bricks, NamedTextColor.GREEN)).build();
//        layout = new ComponentSidebarLayout(title, lines);
//        layout.apply(sb);
//    }

    private @NotNull SidebarAnimation<Component> createGradientAnimation(@NotNull Component text) {
        float step = 1f / 8f;
        TagResolver.Single textPlaceholder = Placeholder.component("text", text);
        List<Component> frames = new ArrayList<>((int) (2f / step));
        float phase = -1f;

        SGM.getInstance().getCLogger().log("Creating gradient animation");

        while (phase <= 1) {
            String gradientTag = String.format("<gradient:yellow:gold:%.2f>", phase);
            Component frame = MiniMessage.miniMessage().deserialize(gradientTag + "<text>", textPlaceholder);
            frames.add(frame);
            SGM.getInstance().getCLogger().log("Phase: " + phase + ", Tag: " + gradientTag);
            phase += step;
        }

        SGM.getInstance().getCLogger().log("Total frames: " + frames.size());
        return new CollectionSidebarAnimation<>(frames);
    }

    private Location parseString(String string) {
        String[] coords = string.trim().split(",");
        if (coords.length == 3) {
            return new Location(gameWorld, Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), Integer.parseInt(coords[2]));
        } else {
            return null;
        }
    }

    // Attempts to load the game's options as a YamlConfiguration
    // before processing each section in the same order as the file.
    private boolean applyOptions(String worldName) {
        File configFile = new File(LevelManager.mapPath + worldName + "/sgm.yml");
        SGM.getInstance().getCLogger().log(configFile.getPath());
        SGM.getInstance().getCLogger().log(configFile.getAbsolutePath());
        YamlConfiguration config = YamlConfiguration.loadConfiguration(configFile);

        // Set map information
        ConfigurationSection mapInfoSection = config.getConfigurationSection("mapInfo");
        if (mapInfoSection != null) {
            this.bricks = mapInfoSection.getInt("brickCount");
            this.territoryLevel = mapInfoSection.getInt("territory");
            this.boundsLevel = mapInfoSection.getInt("boundary");
            this.spawnLoc = parseString(mapInfoSection.getString("spawn"));
        } else {
            SGM.getInstance().getCLogger().error("Invalid map info. Please fix before trying again.");
            return false;
        }

        // Process team information
        List<String> teamsToProcess = Arrays.asList("red", "blue");

        for (String teamName : teamsToProcess) {
            ConfigurationSection team = config.getConfigurationSection("teams." + teamName);
            if (team != null) {
                List<String> brickSpawns = team.getStringList("brickSpawns");
                List<String> playerSpawnArea = team.getStringList("playerSpawnArea");

                // Get brick locations
                if (brickSpawns.size() >= bricks) {
                    for (String rawBrickLoc : brickSpawns) {
                        Location brickLoc = parseString(rawBrickLoc);
                        if (teamName.equals("red")) {
                            this.redBrickSpawns.add(brickLoc);
                        } else if (teamName.equals("blue")) {
                            this.blueBrickSpawns.add(brickLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid brick spawns. Please fix before trying again.");
                    return false;
                }

                // Hopefully that works!
                if (playerSpawnArea.size() == 2) {
                    for (String rawSpawnLoc : playerSpawnArea) {
                        Location spawnLoc = parseString(rawSpawnLoc);
                        if (teamName.equals("red")) {
                            this.redSpawnArea.add(spawnLoc);
                        } else if (teamName.equals("blue")) {
                            this.blueSpawnArea.add(spawnLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid spawn area. Please fix before trying again.");
                    return false;
                }
            } else {
                SGM.getInstance().getCLogger().error("Invalid teams. Please fix before trying again.");
                return false;
            }
        }
        return true;
    }

    @Override
    public void start() {
        if (state == GameState.READY) {
            state = GameState.STARTED;
            sendMessageToAllPlayers("The game has started!");
            for (Player player : players) {
                player.teleport(this.spawnLoc);
                sb.addPlayer(player);
            }
            for (int i = 0; i <= bricks; i++) {
                for (Location brickLoc : redBrickSpawns) {
                    placeBrick(brickLoc, "red");
                }
                for (Location brickLoc : blueBrickSpawns) {
                    placeBrick(brickLoc, "blue");
                }
            }
            BukkitTask task = Bukkit.getScheduler().runTaskTimer(SGM.getInstance(), this::onTick, 0L, 1L);
            this.taskId = task.getTaskId();
        }
    }

    public void addPlayerToTeam(Player player, boolean isRedTeam) {
        //TODO: make this shit fuckin better bor
        if (isRedTeam) {
            redTeam.defaultDisplay().addEntry(player.getName());
        } else {
            blueTeam.defaultDisplay().addEntry(player.getName());
        }
        teamManager.addPlayer(player);
    }

    public void removePlayerFromTeams(Player player) {
        redTeam.defaultDisplay().removeEntry(player.getName());
        blueTeam.defaultDisplay().removeEntry(player.getName());
        teamManager.removePlayer(player);
    }

    public int getAlivePlayersCount() {
        return (int) players.stream().filter(Player::isOnline).count();
    }

    public int getTeamSize(ScoreboardTeam team) {
        return team.defaultDisplay().entries().size();
    }

    private void placeBrick(Location location, String team) {
        Block brick = location.getBlock();
        if (Objects.equals(team, "red")) {
            brick.setType(Material.RED_WOOL);
        }
        if (Objects.equals(team, "blue")) {
            brick.setType(Material.BLUE_WOOL);
        }
        SGM.getInstance().getCLogger().log("Placed " + brick.getType() + " at " + location);
    }

    @Override
    public void stop() {
        // TODO: make it wait before unloading world
        // Stop game logic
        state = GameState.ENDING;
        Player winner;
        if (players.size() <= 1) {
            winner = players.get(0);
            winner.sendMessage("Congratulations on winning the game!");
        }
        players.clear(); // Clear player list
        GameManager.getActiveGames().remove(uuid);
        HandlerList.unregisterAll(this);
        for (Player player : gameWorld.getPlayers()) {
            player.teleport(Bukkit.getServer().getWorlds().get(0).getSpawnLocation());
        }
        Bukkit.unloadWorld(gameWorld, false);
        if (teamManager != null) {
            teamManager.close();
        }
        if (sb != null) {
            sb.close();
        }
        Bukkit.getScheduler().cancelTask(this.taskId);
    }

    @Override
    public void addPlayer(Player player) {
        players.add(player);
//        sb.addPlayer(player);
        player.teleport(this.spawnLoc);
        sendMessageToAllPlayers(messages.getString("system.player.join").replace("%player%", player.getName()));
    }

    @Override
    public void removePlayer(Player player) {
        players.remove(player);
        sb.removePlayer(player);
        sendMessageToAllPlayers(messages.getString("system.player.left").replace("%player%", player.getName()));
    }

    @EventHandler
    void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onWorldChange(PlayerChangedWorldEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player) && state == GameState.STARTED) {
            sendMessageToAllPlayers(player.getName() + " was eliminated!");
            event.setDeathMessage("");
            removePlayer(player);
        }
    }

    @Override
    public List<Player> getPlayers() {
        return players;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public UUID getGameId() {
        return uuid;
    }

    @Override
    public GameState getState() {
        return state;
    }

    @Override
    public World getWorld() {
        return gameWorld;
    }

    @Override
    public void onTick() {
        for (Player player : gameWorld.getPlayers()) {
            if (!players.contains(player)) {
                sendMessageToAllPlayers(player.getName() + " IS A IMPOSTER");
            }
        }
        if (state == GameState.STARTED && players.size() <= 1) {
            this.stop();
        }
        try {
            titleAnimation.nextFrame();
            layout.apply(sb);
        } catch (IllegalStateException ignored) {
        }
    }

}