package xyz.twovb.sgm.guis;

/*
 * Created by 2vb - 5/7/2024
 */

import com.github.stefvanschie.inventoryframework.gui.GuiItem;
import com.github.stefvanschie.inventoryframework.gui.type.ChestGui;
import com.github.stefvanschie.inventoryframework.pane.OutlinePane;
import lombok.Getter;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import xyz.twovb.sgm.SGM;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.List;
import java.util.logging.Level;

public class MapGui {

    @Getter
    private final ChestGui gui;

    private final String game;
    private OutlinePane mapPane;

    public MapGui(String game) {
        this.game = game;
        ChestGui loadedGui = null;
        try {
            File guiFile = new File(SGM.getInstance().getDataFolder(), "guis/mapgui.xml");
            if (guiFile.exists()) {
                byte[] fileContent = Files.readAllBytes(guiFile.toPath());
                InputStream inputStream = new ByteArrayInputStream(fileContent);
                loadedGui = ChestGui.load(this, inputStream);
                mapPane = new OutlinePane(1, 1, 7, 1); // Define your pane layout
                loadedGui.addPane(mapPane);
                addMaps();
            } else {
                SGM.getInstance().getLogger().log(Level.WARNING, "Could not find mapgui.xml file.");
            }
        } catch (Exception e) {
            SGM.getInstance().getLogger().log(Level.SEVERE, "Error loading mapgui.xml", e);
        }
        gui = loadedGui;
    }

    public void globalClick(InventoryClickEvent event) {
        event.setCancelled(true);
    }

    public void addMaps() {
        List<String> enabledMaps = SGM.getInstance().getLevelManager().enabledMaps.get(this.game);
        if (enabledMaps != null) {
            for (String mapName : enabledMaps) {
                // Create button for each map
                ItemStack mapItem = new ItemStack(Material.MAP);
                ItemMeta mapMeta = mapItem.getItemMeta();
                mapMeta.setDisplayName(mapName);
                mapItem.setItemMeta(mapMeta);

                mapPane.addItem(new GuiItem(mapItem, event -> {
                    // Handle click event if needed
                    // Example: navigate to the map or show map details
                     Player player = (Player) event.getWhoClicked();
                    SGM.getInstance().getGameManager().initGame(this.game, player, mapName);
                    player.closeInventory();
//                     player.sendMessage("Clicked on map: " + mapName);
                }));
            }
        } else {
            SGM.getInstance().getLogger().log(Level.WARNING, "No maps found for game: " + game);
        }
    }
}
